@@ -53,6 +53,7 @@
 #define R6 "%r9"
 #define RSP "%rsp"
 #define RBP "%rbp"
+#define Nlseek    "$8"
 #define Nmmap     "$9"
 #define Nmunmap   "$11"
 #define Nread     "$0"
@@ -99,8 +100,9 @@
 #define R6 "%ebp"
 #define RSP "%esp"
 #define RBP "%ebp"
-//#define Nmmap     "$90" don't use this, this requires a pointer to the args
-#define Nmmap     "$192" 
+//#define Nmmap    "$90" don't use this, this requires a pointer to the args
+#define N_llseek  "$140"
+#define Nmmap     "$192"
 #define Nmunmap   "$91"
 #define Nread     "$3"
 #define Nwrite    "$4"
@@ -109,6 +111,9 @@
 #define Nfcntl    "$55"
 #define Nopen     "$5"
 #define Ndup2     "$63"
+#define Nrt_sigaction   "$174"
+#define Nsetuid   "$23"
+#define Nsetgid   "$46"
 
 /**********************************************************************************************
    since x86 is a fucking architecture for linux, this is the socketcall request,
@@ -121,9 +126,6 @@
 #define Nconnect  "$3"
 #define Nlisten   "$4"
 #define Naccept   "$5"
-#define Nrt_sigaction   "$174"
-#define Nsetuid   "$23"
-#define Nsetgid   "$46"
 #define DO_SYSCALL "int $0x80\n\t"
 #define DO_SOCKETCALL   "mov %ebp,4092(%esp)\n\t" \
                         "mov %edi,4088(%esp)\n\t"  \
@@ -166,7 +168,7 @@ hook_info;
 **********************************************************************************************/
 hook_info *thiz;
 hook_info *stoppedpr = NULL;
-int vb = 3;
+int vb = 1;
 int stopped = 0;
 int debug = -1;
 
@@ -319,7 +321,7 @@ void fillbuf_receive_fd(hook_info *h, const char ***syscalls,
             /* fetch the new file descriptor */
             "mov "L2"("RSP"),"R0"\n\t"      /* pointer to the fd */
 #ifdef __x86_64__
-            "movslq ("R0"),"R5"\n\t"        /* passed fd (it is an int32) */
+            "movslq ("R0"),"R5"\n\t"        /* passed fd (it is an int32, convert it to 64bits) */
 #else
             "mov ("R0"),"R5"\n\t"           /* passed fd */
 #endif
@@ -597,6 +599,60 @@ void fillbuf_close(hook_info *h, const char ***syscalls, long fd1)
     *((long*)(h->stack)) = fd1;
 }
 
+/**********************************************************************************************
+    close fd1
+**********************************************************************************************/
+void fillbuf_tell(hook_info *h, const char ***syscalls, long fd1)
+{
+    static const char * __syscalls[] = {
+#ifdef __i386__
+            "_llstat" };
+#else
+            "lstat" };
+#endif
+    *syscalls = __syscalls;
+
+    if(!h->buf)
+    {
+    from5:
+        asm volatile(
+            "mov $0,"RBP"\n\t"
+
+            /* fcntl(fd1,F_GETFD) */
+            "mov ("RSP"),"R1"\n\t"            /* fd to _llseek */
+#ifdef __i386__
+            "mov $0,"R2"\n\t"                 /* hi_pos */
+            "mov $0,"R3"\n\t"                 /* lo_pos */
+            "mov "RSP","R4"\n\t"              /* where to put the result */
+            "mov $1,"R5"\n\t"                 /* SEEK_CUR */
+            "mov "N_llseek","R0"\n\t"
+            DO_SYSCALL                         /* _llseek */
+#else /* AMD64 */
+            "mov $0,"R2"\n\t"                 /* pos */
+            "mov $1,"R3"\n\t"                 /* SEEK_CUR */
+            "mov "Nlseek","R0"\n\t"
+            DO_SYSCALL                         /* _llseek */
+            "mov "RAX",("RSP")\n\t"
+#endif
+            "inc "RBP"\n\tcmp $-127,"R0"\n\tjae .end55\n\t"
+
+            "mov $0,"R0"\n\t"
+            ".end55:\n\t"
+            "int3\n\t"
+        );
+    to5:;
+    }
+
+    /*
+    Yes, i'm using gcc extensions to do this.
+    Too bad you won't be able to compile this with
+    your new copy of MS VisualC Professional Power
+    Edition! :-)
+    */
+    memcpy(h->buf,&&from5,&&to5-&&from5);
+    *((long*)(h->stack)) = fd1;
+}
+
 /**********************************************************************************************
     chdir
 **********************************************************************************************/
@@ -1300,8 +1356,6 @@ hook_info *attach(int pid)
     /* get register */
     ptrace(PTRACE_GETREGS, h->pid, 0, &(h->oldreg));
     h->newreg = h->oldreg;
-    
-    fprintf(stderr,"EFLAGS=%lx\n", h->oldreg.eflags );
 
     /* get an executable and a writable page (from /proc) */
     char buf[512];
@@ -1316,7 +1370,8 @@ hook_info *attach(int pid)
             h->newreg.rip = a;
             sset |= 1;
         }
-        if(((sset&2)==0) && (sscanf(buf,"%lx-%lx rw-p %lx",&a,&b,&c)==3))
+        if(((sset&2)==0) && ( (sscanf(buf,"%lx-%lx rw-p %lx",&a,&b,&c)==3)
+                        || (sscanf(buf,"%lx-%lx rwxp %lx",&a,&b,&c)==3) ))
         {
             h->newreg.rsp = a;
             sset |= 2;
@@ -1607,6 +1662,37 @@ cleanup:
     return retv;
 }
 
+int tell_fd(int fd)
+{
+    __label__ cleanup;
+    int retv = 0;
+    int i;
+    loff_t result;
+
+    if(vb>=1)
+        fprintf(stderr,"OK, querying position of fd %d!\n",fd);
+
+    const char **sys;
+    fillbuf_tell(thiz, &sys, fd);
+    replace_code(thiz);
+
+    if(vb>=2)fprintf(stderr," ->(%d) fd %d beeing closed:\n",thiz->pid,fd);
+    if(debug-- == 0){ debug_detach(thiz); exit(1); }
+    if(!run_to(thiz,sys)) goto cleanup;
+
+    if(debug-- == 0){ debug_detach(thiz); exit(1); }
+    if(vb>=1)fprintf(stderr,"Everythig seems to be all right.\n");
+
+    for(i=0;i<sizeof(loff_t)/sizeof(long);i++)
+        ((long*)&result)[i] = ptrace(PTRACE_PEEKTEXT, thiz->pid, ((char*)thiz->newreg.rsp)+i*sizeof(long), 0);
+    printf("Position of fd %d is: %lld\n", fd, result);
+
+    retv = 1;
+
+cleanup:
+    return retv;
+}
+
 int connect_fd(int fd,const char *host,int port)
 {
     __label__ cleanup;
@@ -2063,13 +2149,13 @@ int restore_mem(const char *filename)
     if(vb>=2)fprintf(stderr," ->(%d)unmapping %lx:\n",thiz->pid, myaddr);
     if(debug-- == 0){ debug_detach(thiz); exit(1); }
     if(!run_to(thiz,sys))goto cleanup;
-    
+
     /* hola!, set new regs! */
     ptrace(PTRACE_SETREGS, thiz->pid, 0, &(thiz->oldreg));
-    
+
     ptrace(PTRACE_GETREGS, thiz->pid, 0, &(thiz->oldreg));
     fprintf(stderr,"EFLAGS=%lx\n", thiz->oldreg.eflags );
-    
+
     if(debug-- == 0){ debug_detach(thiz); exit(1); }
     if(vb>=1)fprintf(stderr,"Everythig seems to be all right.\n");
                 retv = 1;
@@ -2108,6 +2194,7 @@ void usage()
         "    \"{fd}>>{filename}\"      : open file in append mode\n"
         "    \"{fd}<>{filename}\"      : open file for reading and writing\n"
         "    \"{fd}<<{filename}\"      : open file for reading only\n"
+        "    \"tell={fd}\"             : query position of file descriptor\n"
         "    \"chdir={dirname}\"       : change current directory\n"
         "    \"sigign={sig}\"          : ignore signal\n"
         "    \"sigdfl={sig}\"          : set signal to default\n"
@@ -2205,6 +2292,8 @@ int main(int argc, char *argv[])
             retv = fd_to_file(a1,fl,O_ACCMODE,0);
         else if(sscanf(argv[i],"%d>&%16s",&a1,fl)==2 && !strcmp(fl,"-"))
             retv = close_fd(a1);
+        else if(sscanf(argv[i],"tell=%d",&a1)==1)
+            retv = tell_fd(a1);
         else if(sscanf(argv[i],"sigign=%d",&a1)==1)
             retv = set_signal(a1,1);
         else if(sscanf(argv[i],"sigdfl=%d",&a1)==1)
